/*******************************************************************************************/
/**                                                                                       **/
/** COPYRIGHT (C) 2019,  Systemyde International Corporation, ALL RIGHTS RESERVED         **/
/**                                                                                       **/
/** timer chip core                                                   Rev 0.0  08/02/2019 **/
/**                                                                                       **/
/*******************************************************************************************/
module tim_core (c100_pls, data_drv, data_out, isa_drv, fi_drv, fi_out, data_bus, isa_bus,
                 osc_in, ph1, ph2, pwr_upb, pwob, sync);

  input         data_bus;      /* memory/peripheral data bus                          bus  */
  input         isa_bus;       /* instruction address/data bus                        bus  */
  input         osc_in;        /* 32768Hz clock input                                 brd  */
  input         ph1;           /* clock phase 1                                       GB   */
  input         ph2;           /* clock phase 2                                       GB   */
  input         pwr_upb;       /* power on reset (inverted)                           GB   */
  input         pwob;          /* power on (inverted)                                 GB   */
  input         sync;          /* synchronization                                     bus  */

  output        c100_pls;      /* 100 Hz clock                                        test */
  output        data_drv;      /* memory/peripheral data bus                          bus  */
  output        data_out;      /* memory/peripheral data bus                          bus  */
  output        isa_drv;       /* instruction address/data bus                        bus  */
  output        fi_drv;        /* flag bus                                            bus  */
  output        fi_out;        /* flag bus                                            bus  */

  /*****************************************************************************************/
  /* signal declarations                                                                   */
  /*****************************************************************************************/
  wire        a_hz100,      b_hz100;                       /* channel 100Hz                */
  wire        a_hz100g,     b_hz100g;                      /* gated, channel 100Hz         */
  wire        aalrm_nxt,    balrm_nxt;                     /* alarm register input         */
  wire        acc_nxt;                                     /* accuracy register input      */
  wire        acctmr_cry;                                  /* accuracy inc carry           */
  wire        acctmr_inc;                                  /* accuracy inc next            */
  wire        acctmr_tc;                                   /* accuracy terminal count      */
  wire        acctmrdadj_cry;                              /* accuracy dec adj carry       */
  wire        aclk_0cry,    bclk_0cry;                     /* clock inc force carry in 0   */
  wire        aclk_1cry,    bclk_1cry;                     /* clock inc force carry in 1   */
  wire        aclk_cry,     bclk_cry;                      /* clock inc carry out          */
  wire        aclk_cry_in,  bclk_cry_in;                   /* clock inc carry in           */
  wire        aclk_inc,     bclk_inc;                      /* clock inc next               */
  wire        aclk_mat,     bclk_mat;                      /* clock alarm match            */
  wire        aclkdadj_cry, bclkdadj_cry;                  /* clock dec adj carry          */
  wire        adjust_pls;                                  /* synced adjust pulse          */
  wire        alma_res,     almb_res;                      /* alarm reset                  */
  wire        ascr_nxt,     bscr_nxt;                      /* scratch register input       */
  wire        astatus_out,  bstatus_out;                   /* status reg read data         */
  wire        c10240_pls;                                  /* 10240 Hz clock               */
  wire        c5600_pls;                                   /* 5600 Hz clock                */
  wire        c6400_pls;                                   /* 6400 Hz clock                */
  wire        cha_sel,      chb_sel;                       /* channel selects              */
  wire        daddc_pls;                                   /* dadd=c inst pulse            */
  wire        data_drv,     data_out;                      /* memory/peripheral data bus   */
  wire        dtza_res,     dtzb_res;                      /* dtz reset                    */
  wire        dtzit_res;                                   /* dtzit reset                  */
  wire        en_aalrm,     en_balrm;                      /* enable alarm register        */
  wire        en_aclk,      en_bclk;                       /* enable clock register        */
  wire        en_acc;                                      /* enable accuracy register     */
  wire        en_accmatch;                                 /* accuracy match enable        */
  wire        en_acctmr;                                   /* enable accuracy timer inc    */
  wire        en_ascr,      en_bscr;                       /* enable scratch register      */
  wire        en_itmr;                                     /* enable interval timer inc    */
  wire        en_tmatch;                                   /* interval timer match enable  */
  wire        en_tmr;                                      /* enable interval timer reg    */
  wire        fi_drv, fi_out;                              /* flag bus                     */
  wire        i_hz100;                                     /* itimer 100Hz                 */
  wire        isa_drv;                                     /* instr addr/data bus          */
  wire        itmr_cry;                                    /* itimer inc carry             */
  wire        itmr_inc;                                    /* itimer inc next              */
  wire        itmr_tc;                                     /* itimer terminal count        */
  wire        itmrdadj_cry;                                /* itimer dec adj carry         */
  wire        ld_cycle;                                    /* next cycle                   */
  wire        ld_inst;                                     /* latch instruction            */
  wire        pfadc_pls;                                   /* pfad=c inst pulse            */
  wire        ph2m;                                        /* muxed phase 2 clock          */
  wire        pus_res;                                     /* pus reset                    */
  wire        rd_aalrm,     rd_balrm;                      /* channel read alrm            */
  wire        rd_aclk,      rd_bclk;                       /* channel read clk             */
  wire        rd_ahclk,     rd_bhclk;                      /* channel read/hold clk        */
  wire        rd_aonly,     rd_bonly;                      /* channel read clk only        */
  wire        rdacc_inst;                                  /* read factor inst             */
  wire        rdalrm_inst;                                 /* read alarm inst              */
  wire        rdclk_inst;                                  /* read clock inst              */
  wire        rdhclk_inst;                                 /* read/hold clock inst         */
  wire        rdscr_inst;                                  /* read scratch inst            */
  wire        rdtmr_inst;                                  /* read interval timer inst     */
  wire        srvc_drv;                                    /* service request drv enable   */
  wire        srvc_req;                                    /* service request              */
  wire        start_idl;                                   /* trigger idle state           */
  wire        start_run;                                   /* trigger run state            */
  wire        state_c80;                                   /* states c-8-0                 */
  wire        state_c80p;                                  /* states c-8-0, +1             */
  wire        state_ec;                                    /* states e-c                   */
  wire        state_ec80;                                  /* states e-c-8-0               */
  wire        state_ec801;                                 /* states e-c-8-0-1             */
  wire        tmr_nxt;                                     /* itimer reg input             */
  wire        vpwo;                                        /* virtual pwo                  */
  wire        wr_aalrm,     wr_balrm;                      /* channel write alrm           */
  wire        wr_aclk,      wr_bclk;                       /* channel write clk            */
  wire        wr_acclk,     wr_bcclk;                      /* channel write/correct clk    */
  wire        wracc_inst;                                  /* write accuracy inst          */
  wire        wralm_inst;                                  /* write en/dis alarm inst      */
  wire        wralrm_inst;                                 /* write alarm inst             */
  wire        wrcclk_inst;                                 /* write/correct clock inst     */
  wire        wrclk_inst;                                  /* write clock inst             */
  wire        writim_inst;                                 /* write itimer inst            */
  wire        wrptr_inst;                                  /* write pointer inst           */
  wire        wrscr_inst;                                  /* write scratch inst           */
  wire        wrstat_inst;                                 /* write status inst            */
  wire        wrstst_inst;                                 /* write start/stop inst        */
  wire        wrtmr_inst;                                  /* write interval timer inst    */
  wire        x_hz100;                                     /* acctimer 100Hz               */
  wire  [2:0] aclk_nxt,     bclk_nxt;                      /* clock reg next               */
  wire  [3:0] acctmr_nxt;                                  /* accuracy next                */
  wire  [3:0] itmr_nxt;                                    /* itimer next                  */

  reg         accen_reg;                                   /* accuracy timer enable        */
  reg         accmatch_reg;                                /* accuracy match compare       */
  reg         acctmr_cry_reg;                              /* accuracy latched carry bit   */
  reg         acctmrtc_reg;                                /* accuracy latched term cnt    */
  reg         aclk_cry_reg, bclk_cry_reg;                  /* clock latched carry bit      */
  reg         aclkhld_reg,  bclkhld_reg;                   /* hold increment op            */
  reg         aclkinc2_reg, bclkinc2_reg;                  /* increment by 2               */
  reg         alaen_reg,    alben_reg;                     /* alarm enable registers       */
  reg         alma_reg,     almb_reg;                      /* alarm match                  */
  reg         amatch_reg,   bmatch_reg;                    /* alarm match compare          */
  reg         awake_reg;                                   /* in sync with awake cpu       */
  reg         c100_pls;                                    /* 100 Hz clock                 */
  reg         c16384_pls;                                  /* 16384 Hz clock               */
  reg         ckaen_reg,    ckben_reg;                     /* clock enable registers       */
  reg         cregn_reg,    regnc_reg;                     /* latched inst decodes         */
  reg         daddc_reg;                                   /* dadd=c inst                  */
  reg         data_cha,     data_chb;                      /* read data mux outputs        */
  reg         data_reg;                                    /* ph1 sampled data signal      */
  reg         dtza_reg,     dtzb_reg;                      /* clock dec thru zero          */
  reg         dtzit_reg;                                   /* interval tmr dec thru zero   */
  reg         enabl_reg;                                   /* chip enable                  */
  reg         isa_reg;                                     /* ph1 sampled isa signal       */
  reg         iten_reg;                                    /* timer enable register        */
  reg         itmr_cry_reg;                                /* itimer latched carry bit     */
  reg         itmrtc_reg;                                  /* itimer latched term cnt      */
  reg         pfadc_reg;                                   /* pfad=c inst                  */
  reg         powoff_inv_reg;                              /* powoff inst (opp polarity)   */
  reg         ptr_inv_reg;                                 /* pointer reg (opp polarty)    */
  reg         pus_inv_reg;                                 /* pwr-up status (opp polarity) */
  reg         state_0, state_1, state_2, state_3;          /* machine state                */
  reg         state_4, state_5, state_6;
  reg         state_8, state_9, state_a, state_b;
  reg         state_c, state_d, state_e;
  reg         state_w;                                     /* wake state                   */
  reg         sync_dly;                                    /* delayed sync signal          */
  reg         sync_reg;                                    /* ph1 sampled sync signal      */
  reg         syndet_reg;                                  /* sync the machine state       */
  reg         t0_reg, t1_reg;                              /* bit identifiers              */
  reg         tmatch_reg;                                  /* interval tmr match compare   */
  reg         updown_reg;                                  /* latched adj dir select       */
  reg         vpwo_reg;                                    /* virtual pwo                  */
  reg   [1:0] c100lat_nxt;                                 /* 100Hz synced                 */
  reg   [2:0] c10240_reg;                                  /* 10240 Hz divider             */
  reg   [2:0] acctmrdadj_out;                              /* accuracy dec adj output      */
  reg   [2:0] aclkdadj_out, bclkdadj_out;                  /* clock dec adj output         */
  reg   [2:0] c5600_reg;                                   /* 5600 Hz divider              */
  reg   [2:0] c6400_reg;                                   /* 6400 Hz divider              */
  reg   [2:0] itmrdadj_out;                                /* itimer dec adj output        */
  reg   [3:0] cycle_reg;                                   /* clock cycle                  */
  reg   [3:0] syncnt_reg;                                  /* sync detector                */
  reg   [5:0] c100_reg;                                    /* 100 Hz divider               */
  reg   [7:0] din_reg;                                     /* address latch                */
  reg   [9:0] iftch_reg;                                   /* instruction assembly         */
  reg   [9:6] lftch_reg;                                   /* latched instruction          */
  reg  [12:0] acc_reg;                                     /* accuracy register            */
  reg  [15:0] acctmr_reg;                                  /* accuracy time register       */
  reg  [19:0] itmr_reg;                                    /* time register                */
  reg  [19:0] tmr_reg;                                     /* interval timer register      */
  reg  [55:0] aclk_reg,     bclk_reg;                      /* clock registers              */
  reg  [55:0] aalrm_reg,    balrm_reg;                     /* alarm registers              */
  reg  [55:0] ascr_reg,     bscr_reg;                      /* scratch registers            */

  /*****************************************************************************************/
  /* generated clocks                                                                      */
  /*****************************************************************************************/
  reg         c5600_raw    /* synthesis syn_preserve=1 */; /* 5600 Hz clock                */
  wire        ph2m_raw     /* synthesis syn_keep=1 */;     /* muxed phase 2 clock          */

  /*****************************************************************************************/
  /* clock domain crossing, clock generation                                               */
  /*****************************************************************************************/
  reg         adj_dn_reg   /* synthesis syn_preserve=1 */; /* adjust prescaler down        */
  reg         adj_up_reg   /* synthesis syn_preserve=1 */; /* adjust prescaler up          */
  reg         adjlat_reg   /* synthesis syn_preserve=1 */; /* adjust synced                */
  reg         hz100_pls    /* synthesis syn_preserve=1 */; /* synced 100Hz pulse           */
  reg         hz100_reg    /* synthesis syn_preserve=1 */; /* 10mS time tick               */
  reg   [1:0] c100lat_reg  /* synthesis syn_preserve=1 */; /* 100Hz synced                 */
  reg   [1:0] idl_ph2_reg  /* synthesis syn_preserve=1 */; /* ph2 sampled idle state       */
  reg   [1:0] idl_reg      /* synthesis syn_preserve=1 */; /* idle state tracker           */
  reg   [1:0] pwo_c56_reg  /* synthesis syn_preserve=1 */; /* c5600 sampled pwo signal     */
  reg   [1:0] pwo_reg      /* synthesis syn_preserve=1 */; /* ph2 sampled pwo signal       */
  reg   [1:0] run_reg      /* synthesis syn_preserve=1 */; /* run state tracker            */
  reg   [2:0] adjsyn_reg   /* synthesis syn_preserve=1 */; /* adjust synchronization       */
  reg   [2:0] c100syn_reg  /* synthesis syn_preserve=1 */; /* 100Hz synchronizer           */

  /*****************************************************************************************/
  /* fpga-specific                                                                         */
  /*****************************************************************************************/
`ifdef SIM_VERSION

  assign      ph2m      = ph2m_raw;
  assign      c5600_pls = c5600_raw;

`else

  SB_GB  PH2M    ( .USER_SIGNAL_TO_GLOBAL_BUFFER(ph2m_raw),
                   .GLOBAL_BUFFER_OUTPUT(ph2m) );

  SB_GB  C5600   ( .USER_SIGNAL_TO_GLOBAL_BUFFER(c5600_raw),
                   .GLOBAL_BUFFER_OUTPUT(c5600_pls) );

`endif

  /*****************************************************************************************/
  /* clock multiplexer: ph2 during run, c5600_pls during idle                              */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwr_upb) begin
    if (pwr_upb) begin
      idl_ph2_reg <= 2'b00;
      pwo_reg     <= 2'b00;
      sync_dly    <= 1'b0;
      end
    else begin
      idl_ph2_reg <= {idl_reg[1], idl_ph2_reg[1]};
      pwo_reg     <= {vpwo, pwo_reg[1]};
      sync_dly    <= sync_reg;
      end
    end

  always @ (posedge c5600_pls or posedge pwr_upb) begin
    if (pwr_upb) pwo_c56_reg <= 2'b00;
    else         pwo_c56_reg <= {vpwo, pwo_c56_reg[1]};
    end


  assign start_idl = state_b && cycle_reg[2];
  assign start_run = !sync && sync_dly;

  always @ (posedge ph2 or posedge pwr_upb) begin
    if (pwr_upb) run_reg <= 2'b00;
    else         run_reg <= {((run_reg[1] && !run_reg[0]) ||
                              (!run_reg[1] && idl_ph2_reg[0]) ||
                              (&run_reg && !start_run)),
                             ((run_reg[1] && !run_reg[0] && !idl_ph2_reg[0]) ||
                              (&run_reg && !start_run))};
    end

  always @ (posedge c5600_pls or posedge pwr_upb) begin
    if (pwr_upb) idl_reg <= 2'b00;
    else         idl_reg <= {((idl_reg[1] && !idl_reg[0]) ||
                              (!idl_reg[1] && !pwo_c56_reg[0] && pus_inv_reg)),
                             (idl_reg[1] && !idl_reg[0] && pwo_c56_reg[0] && start_idl)};
    end

  assign ph2m_raw = (ph2 || !pwo_reg[0] || run_reg[1] || idl_reg[1]) &&
                    (c5600_pls || idl_reg[0] || ~|idl_reg);

  always @ (posedge ph2m or posedge pwr_upb) begin
    if (pwr_upb) awake_reg <= 1'b0;
    else         awake_reg <= pwo_reg[0] && !run_reg[1] && !idl_reg[1];
    end

  /*****************************************************************************************/
  /* virtual pwo                                                                           */
  /*****************************************************************************************/
  always @ (posedge ph1 or posedge pwr_upb) begin
    if (pwr_upb) vpwo_reg <= 1'b0;
    else begin
      vpwo_reg <= !(!sync && sync_dly && ~|(iftch_reg ^ 10'h060)) && (pwob || vpwo_reg);
      end
    end

  assign vpwo = vpwo_reg && !pwob;

  /*****************************************************************************************/
  /* sample bus signals                                                                    */
  /*****************************************************************************************/
  always @ (negedge ph1 or posedge pwr_upb) begin
    if (pwr_upb) begin
      data_reg <= 1'b0;
      isa_reg  <= 1'b0;
      sync_reg <= 1'b0;
      end
    else begin
      data_reg <= data_bus;
      isa_reg  <= isa_bus;
      sync_reg <= sync;
      end
    end

  /*****************************************************************************************/
  /* data input latch                                                                      */
  /*****************************************************************************************/
  always @ (posedge ph2m or posedge pwob) begin
    if          (pwob) din_reg <= 8'h00;
    else if (state_ec) din_reg <= {(awake_reg && data_reg), din_reg[7:1]};
    end

  /*****************************************************************************************/
  /* instruction register and decodes                                                      */
  /*****************************************************************************************/
  always @ (posedge ph2m or posedge pwob) begin
    if          (pwob) iftch_reg <= 10'h000;
    else if (sync_reg) iftch_reg <= {(awake_reg && isa_reg), iftch_reg[9:1]};
    end

  always @ (posedge ph2m or posedge pwob) begin
    if (pwob) begin
      cregn_reg      <= 1'b0;
      daddc_reg      <= 1'b0;
      lftch_reg      <= 4'h0;
      pfadc_reg      <= 1'b0;
      powoff_inv_reg <= 1'b0;
      regnc_reg      <= 1'b0;
      end
    else if (ld_inst) begin
      cregn_reg      <= ~|(iftch_reg[5:0] ^ 6'b111000);
      daddc_reg      <= ~|(iftch_reg ^ 10'h270);
      lftch_reg      <= iftch_reg[9:6];
      pfadc_reg      <= ~|(iftch_reg ^ 10'h3f0);
      powoff_inv_reg <=  |(iftch_reg ^ 10'h060);
      regnc_reg      <= ~|(iftch_reg[5:0] ^ 6'b101000);
      end
    end

  /*****************************************************************************************/
  /* instruction register and decodes                                                      */
  /*****************************************************************************************/
  assign rdclk_inst  = enabl_reg && cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0000);
  assign rdhclk_inst = enabl_reg && cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0001);
  assign rdalrm_inst = enabl_reg && cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0010);
  assign rdacc_inst  = enabl_reg && cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0011) && chb_sel;
  assign rdscr_inst  = enabl_reg && cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0100);
  assign rdtmr_inst  = enabl_reg && cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0101);

  assign wrclk_inst  = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0000);
  assign wrcclk_inst = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0001);
  assign wralrm_inst = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0010);
  assign wrstat_inst = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0011) && cha_sel;
  assign wracc_inst  = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0011) && chb_sel;
  assign wrscr_inst  = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0100);
  assign wrtmr_inst  = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0101);
  assign writim_inst = enabl_reg && regnc_reg && ~|(lftch_reg[9:8] ^ 2'b01) && lftch_reg[6];
  assign wralm_inst  = enabl_reg && regnc_reg && ~|(lftch_reg[9:7] ^ 3'b101);
  assign wrstst_inst = enabl_reg && regnc_reg && ~|(lftch_reg[9:7] ^ 3'b110);
  assign wrptr_inst  = enabl_reg && regnc_reg && ~|(lftch_reg[9:7] ^ 3'b111);

  assign alma_res    = wrstat_inst && !din_reg[0];
  assign dtza_res    = wrstat_inst && !din_reg[1];
  assign almb_res    = wrstat_inst && !din_reg[2];
  assign dtzb_res    = wrstat_inst && !din_reg[3];
  assign dtzit_res   = wrstat_inst && !din_reg[4];
  assign pus_res     = wrstat_inst && !din_reg[5];

  /*****************************************************************************************/
  /* chip enable decode                                                                    */
  /*****************************************************************************************/
  assign daddc_pls = ld_inst && daddc_reg;
  assign pfadc_pls = ld_inst && pfadc_reg;

  always @ (posedge ph2m or posedge pwob) begin
    if                        (pwob) enabl_reg <= 1'b0;
    else if (daddc_pls || pfadc_pls) enabl_reg <= pfadc_pls && ~|(din_reg ^ 8'hfb);
    end

  /*****************************************************************************************/
  /* pointer register                                                                      */
  /*****************************************************************************************/
  always @ (posedge ph2m or posedge pwr_upb) begin
    if                    (pwr_upb) ptr_inv_reg <= 1'b0;
    else if (ld_inst && wrptr_inst) ptr_inv_reg <= !lftch_reg[6];
    end

  assign cha_sel = !ptr_inv_reg;
  assign chb_sel =  ptr_inv_reg;

  /*****************************************************************************************/
  /* power up status bit                                                                   */
  /*****************************************************************************************/
  always @ (posedge ph2m or posedge pwr_upb) begin
    if      (pwr_upb) pus_inv_reg <= 1'b0;
    else if (ld_inst) pus_inv_reg <= pus_res || pus_inv_reg;
    end

  /*****************************************************************************************/
  /* bus synchronization                                                                   */
  /*****************************************************************************************/
  always @ (posedge ph2m or posedge pwob) begin
    if (pwob) begin
      syncnt_reg <= 4'b0000;
      syndet_reg <= 1'b0;
      end
    else begin
      syncnt_reg <= (sync_reg) ? (syncnt_reg + 1'b1) : 4'b0000;
      syndet_reg <= !sync_reg && ~|(syncnt_reg ^ 4'b1010) && awake_reg && powoff_inv_reg;
      end
    end

  /*****************************************************************************************/
  /* bus state machine                                                                     */
  /*****************************************************************************************/
  /*                                                                                       */
  /*   state_x         eeeecccc88880000111122225555aaaa4444999933336666ddddbbbb            */
  /*   cycle_reg       12481248124812481248124812481248124812481248124812481248            */
  /*   ld_cycle        ...1...1...1...1...1...1...1...1...1...1...1...1...1...1            */
  /*   ld_inst         .......................................................1            */
  /*   sync            ............................................1111111111..            */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2m or posedge pwr_upb) begin
    if (pwr_upb) cycle_reg <= 4'b0000;
    else         cycle_reg <= {( !cycle_reg[3]   && cycle_reg[2] && !syndet_reg),
                               (~|cycle_reg[3:2] && cycle_reg[1] && !syndet_reg),
                               (~|cycle_reg[3:1] && !syndet_reg),
                               (cycle_reg[3] || syndet_reg)};
    end

  always @ (posedge ph2m or posedge pwr_upb) begin
    if (pwr_upb) begin
      state_w <= 1'b0;
      state_e <= 1'b0;
      state_c <= 1'b0;
      state_8 <= 1'b0;
      state_0 <= 1'b0;
      state_1 <= 1'b0;
      state_2 <= 1'b0;
      state_5 <= 1'b0;
      state_a <= 1'b0;
      state_4 <= 1'b0;
      state_9 <= 1'b0;
      state_3 <= 1'b0;
      state_6 <= 1'b0;
      state_d <= 1'b0;
      state_b <= 1'b0;
      end
    else if (ld_cycle) begin
      state_w <= 1'b1;
      state_e <= state_b ||  syndet_reg || !state_w;
      state_c <= state_e && !syndet_reg;
      state_8 <= state_c && !syndet_reg;
      state_0 <= state_8 && !syndet_reg;
      state_1 <= state_0 && !syndet_reg;
      state_2 <= state_1 && !syndet_reg;
      state_5 <= state_2 && !syndet_reg;
      state_a <= state_5 && !syndet_reg;
      state_4 <= state_a && !syndet_reg;
      state_9 <= state_4 && !syndet_reg;
      state_3 <= state_9 && !syndet_reg;
      state_6 <= state_3 && !syndet_reg;
      state_d <= state_6 && !syndet_reg;
      state_b <= state_d && !syndet_reg;
      end
    end

  always @ (posedge ph2m or posedge pwr_upb) begin
    if (pwr_upb) begin
      t0_reg <= 1'b0;
      t1_reg <= 1'b0;
      end
    else begin
      t0_reg <= state_b && cycle_reg[3];
      t1_reg <= state_e && cycle_reg[0];
      end
    end

  assign ld_cycle    = cycle_reg[3];
  assign ld_inst     = state_b && cycle_reg[3];
  assign state_c80   = state_c || state_8 || state_0;
  assign state_c80p  = state_c || state_8 || state_0 || (state_1 && cycle_reg[0]);
  assign state_ec    = state_e || state_c;
  assign state_ec80  = state_e || state_c || state_8 || state_0;
  assign state_ec801 = state_e || state_c || state_8 || state_0 || state_1;

  /*****************************************************************************************/
  /* clock dividers                                                                        */
  /*****************************************************************************************/
  always @ (posedge osc_in or posedge pwr_upb) begin
    if (pwr_upb) begin
      c16384_pls <= 1'b0;
      c10240_reg <= 3'b000;
      c6400_reg  <= 3'b000;
      c5600_reg  <= 3'b000;
      c100_reg   <= 6'b000000;
      c100_pls   <= 1'b0;
      c5600_raw  <= 1'b0;
      end
    else begin
      c16384_pls <= !c16384_pls;
      if (c16384_pls) c10240_reg <= c10240_reg + 1'b1;
      if (c10240_pls) c6400_reg  <= c6400_reg + 1'b1;
      if (c6400_pls)  c5600_reg  <= c5600_reg + 1'b1;
      if (c5600_pls)  c100_reg   <= (~|(c100_reg ^ 6'b110111)) ? 6'b000000 :
                                                                 (c100_reg + 1'b1);
      c100_pls   <= c100_reg[5];
      c5600_raw  <= c6400_pls && ~&c5600_reg;
      end
    end

  assign c10240_pls = c16384_pls && (!c10240_reg[2] || (~|c10240_reg[1:0] && !adj_dn_reg) ||
                                    (&c10240_reg[1:0] && adj_up_reg));
  assign c6400_pls  = c10240_pls && (!c6400_reg[2] || &c6400_reg[1:0]);

  /*****************************************************************************************/
  /* clock domain crossing (both directions)                                               */
  /*****************************************************************************************/
  assign adjust_pls = !adjsyn_reg[1] && adjsyn_reg[0];

  always @ (posedge osc_in or posedge pwr_upb) begin
    if (pwr_upb) begin
      adj_dn_reg <= 1'b0;
      adj_up_reg <= 1'b0;
      adjsyn_reg <= 3'b000;
      adjlat_reg <= 1'b0;
      end
    else if (c16384_pls) begin
      if (~|c10240_reg) begin
        adj_dn_reg <=  updown_reg && (adjust_pls || adjlat_reg);
        adj_up_reg <= !updown_reg && (adjust_pls || adjlat_reg);
        end
      adjsyn_reg <= {acctmrtc_reg, adjsyn_reg[2:1]};
      adjlat_reg <= |c10240_reg && (adjust_pls || adjlat_reg);
      end
    end

  always @ (hz100_pls or ld_inst or c100lat_reg) begin
    case ({ld_inst, hz100_pls})
      2'b01:   c100lat_nxt = {c100lat_reg[0], 1'b1};       /* one more pending             */
      2'b10:   c100lat_nxt = {1'b0, c100lat_reg[1]};       /* one less pending             */
      default: c100lat_nxt = c100lat_reg;                  /* no change                    */
      endcase
    end

  always @ (posedge ph2m or posedge pwr_upb) begin
    if (pwr_upb) begin
      c100lat_reg <= 2'b00;
      c100syn_reg <= 3'b000;
      hz100_pls   <= 1'b0;
      hz100_reg   <= 1'b0;
      end
    else begin
      c100lat_reg <= c100lat_nxt;
      c100syn_reg <= {c100_pls, c100syn_reg[2:1]};
      hz100_pls   <= !c100syn_reg[1] && c100syn_reg[0];
      if (ld_inst) hz100_reg <= hz100_pls || |c100lat_reg;
      end
    end

  /*****************************************************************************************/
  /* service requests                                                                      */
  /*****************************************************************************************/
  assign srvc_drv = state_d && enabl_reg;
  assign srvc_req = alma_reg || almb_reg || dtza_reg || dtzb_reg || dtzit_reg;

  assign isa_drv  =  pwob && pus_inv_reg && srvc_req;
  assign fi_drv   = !pwob && pus_inv_reg && awake_reg &&
                     ((state_b && (srvc_req || cycle_reg[3])) || srvc_drv);
  assign fi_out   =  !pwob && pus_inv_reg && awake_reg && ((state_b && cycle_reg[3]) ||
                     (srvc_drv && (!srvc_req || cycle_reg[3])));

  /*****************************************************************************************/
  /* data bus                                                                              */
  /*****************************************************************************************/
  assign data_drv = enabl_reg && cregn_reg && !ph2m;
  assign data_out = data_cha || data_chb;

  /*****************************************************************************************/
  /* read data a                                                                           */
  /*****************************************************************************************/
  always @ (lftch_reg or cha_sel or aalrm_reg or aclk_reg or ascr_reg or state_ec801 or
            astatus_out or tmr_reg) begin
    casex ({lftch_reg[8:6], cha_sel})
      4'b00x1: data_cha = aclk_reg[0];                     /* clock register a             */
      4'b0101: data_cha = aalrm_reg[0];                    /* alarm register a             */
      4'b0111: data_cha = astatus_out;                     /* status                       */
      4'b1001: data_cha = ascr_reg[0];                     /* scratch register a           */
      4'b101x: data_cha = tmr_reg[0] && state_ec801;       /* interval timer               */
      default: data_cha = 1'b0;
      endcase
    end

  /*****************************************************************************************/
  /* read data b                                                                           */
  /*****************************************************************************************/
  always @ (lftch_reg or chb_sel or acc_reg or balrm_reg or bclk_reg or bscr_reg or
            bstatus_out or state_c80p) begin
    casex ({lftch_reg[8:6], chb_sel})
      4'b00x1: data_chb = bclk_reg[0];                     /* clock register b             */
      4'b0101: data_chb = balrm_reg[0];                    /* alarm register b             */
      4'b0111: data_chb = acc_reg[0] && state_c80p;        /* accuracy factor              */
      4'b0110: data_chb = bstatus_out;                     /* status                       */
      4'b1001: data_chb = bscr_reg[0];                     /* scratch register b           */
      default: data_chb = 1'b0;
      endcase
    end

  /*****************************************************************************************/
  /* status register a                                                                     */
  /*****************************************************************************************/
  always @ (posedge ph2m or posedge pwr_upb) begin
    if (pwr_upb) begin
      alaen_reg <= 1'b0;
      ckaen_reg <= 1'b0;
      iten_reg  <= 1'b0;
      end
    else begin
      if (ld_inst && cha_sel &&  wralm_inst) alaen_reg <=  lftch_reg[6];
      if (ld_inst && cha_sel && wrstst_inst) ckaen_reg <=  lftch_reg[6];
      if (ld_inst &&            writim_inst) iten_reg  <= !lftch_reg[7];
      end
    end

  assign astatus_out = (state_e && cycle_reg[0] && alma_reg) ||
                       (state_e && cycle_reg[1] && dtza_reg) ||
                       (state_c && cycle_reg[0] && dtzit_reg) ||
                       (state_c && cycle_reg[1] && !pus_inv_reg) ||
                       (state_c && cycle_reg[2] && ckaen_reg) ||
                       (state_8 && cycle_reg[0] && alaen_reg) ||
                       (state_8 && cycle_reg[2] && iten_reg);

  /*****************************************************************************************/
  /* status register b                                                                     */
  /*****************************************************************************************/
  always @ (posedge ph2m or posedge pwr_upb) begin
    if (pwr_upb) begin
      alben_reg <= 1'b0;
      ckben_reg <= 1'b0;
      end
    else begin
      if (ld_inst && chb_sel &&  wralm_inst) alben_reg <=  lftch_reg[6];
      if (ld_inst && chb_sel && wrstst_inst) ckben_reg <=  lftch_reg[6];
      end
    end

  assign bstatus_out = (state_e && cycle_reg[2] && almb_reg) ||
                       (state_e && cycle_reg[3] && dtzb_reg) ||
                       (state_c && cycle_reg[3] && ckben_reg) ||
                       (state_8 && cycle_reg[1] && alben_reg);

  /*****************************************************************************************/
  /* scratch a register                                                                    */
  /*****************************************************************************************/
  assign en_ascr  = cha_sel && (rdscr_inst || wrscr_inst);
  assign ascr_nxt = (wrscr_inst) ? data_reg : ascr_reg[0];

  always @ (posedge ph2m or posedge pwr_upb) begin
    if      (pwr_upb) ascr_reg <= 56'h00000000000000;
    else if (en_ascr) ascr_reg <= {ascr_nxt, ascr_reg[55:1]};
    end

  /*****************************************************************************************/
  /* scratch b register                                                                    */
  /*****************************************************************************************/
  assign en_bscr  = chb_sel && (rdscr_inst || wrscr_inst);
  assign bscr_nxt = (wrscr_inst) ? data_reg : bscr_reg[0];

  always @ (posedge ph2m or posedge pwr_upb) begin
    if      (pwr_upb) bscr_reg <= 56'h00000000000000;
    else if (en_bscr) bscr_reg <= {bscr_nxt, bscr_reg[55:1]};
    end

  /*****************************************************************************************/
  /* alarm/clock a                                                                         */
  /*****************************************************************************************/
  assign a_hz100  = ckaen_reg && hz100_reg;
  assign a_hz100g = ckaen_reg && hz100_reg && !aclkhld_reg;

  assign rd_aalrm =  cha_sel &&  rdalrm_inst;
  assign rd_aclk  =  cha_sel && (rdhclk_inst || rdclk_inst);
  assign rd_ahclk =  cha_sel &&  rdhclk_inst;
  assign rd_aonly =  rd_ahclk || aclkhld_reg || (rd_aclk && !a_hz100);

  assign wr_aalrm =  cha_sel &&  wralrm_inst;
  assign wr_aclk  =  cha_sel && (wrcclk_inst || wrclk_inst);
  assign wr_acclk =  cha_sel &&  wrcclk_inst;

  assign en_aalrm  = a_hz100g || rd_aalrm || wr_aalrm;
  assign aalrm_nxt = (wr_aalrm) ? data_reg : aalrm_reg[0];
  assign aclk_mat  = a_hz100g &&
                     ~|({aclkdadj_out, aclk_reg[53]} ^ {aalrm_reg[0], aalrm_reg[55:53]});

  always @ (posedge ph2m or posedge pwr_upb) begin
    if (pwr_upb) begin
      aalrm_reg  <= 56'h00000000000000;
      amatch_reg <= 1'b0;
      alma_reg   <= 1'b0;
      end
    else begin
      if (en_aalrm) aalrm_reg  <= {aalrm_nxt, aalrm_reg[55:1]};
      if (ld_cycle) amatch_reg <= alaen_reg && ckaen_reg &&
                                  (ld_inst || (aclk_mat && amatch_reg));
      if (ld_inst)  alma_reg   <= (aclk_mat && amatch_reg) || (!alma_res && alma_reg);
      end
    end

  assign en_aclk  = a_hz100g || rd_aclk || wr_aclk;
  assign aclk_inc = aclk_reg[0] ^  aclk_cry_in;
  assign aclk_cry = aclk_reg[0] && aclk_cry_in;

  assign aclk_nxt = (wr_aclk)  ? {data_reg,    aclk_reg[55:54]} :
                    (rd_aonly) ? {aclk_reg[0], aclk_reg[55:54]} :
                    (ld_cycle) ?  aclkdadj_out :
                                 {aclk_inc,    aclk_reg[55:54]};

  assign aclk_0cry   = wr_aclk || rd_aonly || !a_hz100g ||
                       ( aclkinc2_reg && a_hz100g && t0_reg); 
  assign aclk_1cry   = (!aclkinc2_reg && a_hz100g && t0_reg) ||
                       ( aclkinc2_reg && a_hz100g && t1_reg);
  assign aclk_cry_in = !aclk_0cry && (aclk_1cry || aclk_cry_reg);

  always @ (aclk_cry or aclk_inc or aclk_reg) begin
    casex ({aclk_cry, aclk_inc, aclk_reg[55:54]})
      4'bx101: aclkdadj_out = 3'b000;                      /* for inc operation            */
      4'bx110: aclkdadj_out = 3'b001;                      /* if result >= 1010            */ 
      4'bx111: aclkdadj_out = 3'b010;                      /* then sub 1010 from result    */
      4'b1000: aclkdadj_out = 3'b011;
      default: aclkdadj_out = {aclk_inc, aclk_reg[55:54]}; /* else no change               */
      endcase
    end

  assign aclkdadj_cry = aclk_cry || (aclk_inc && |aclk_reg[55:54]);

  always @ (posedge ph2m or posedge pwr_upb) begin
    if (pwr_upb) begin
      aclk_cry_reg <= 1'b0;
      aclk_reg     <= 56'h00000000000000;
      aclkhld_reg  <= 1'b0;
      aclkinc2_reg <= 1'b0;
      dtza_reg     <= 1'b0;
      end
    else begin
      aclk_cry_reg          <= (ld_cycle) ? aclkdadj_cry : aclk_cry;
      if (en_aclk) aclk_reg <= {aclk_nxt, aclk_reg[53:1]};
      if (ld_inst) begin
        aclkhld_reg  <= !wr_acclk && (rd_ahclk || aclkhld_reg);
        aclkinc2_reg <= (a_hz100 && (rd_ahclk || aclkhld_reg)) ||
                        (!a_hz100g && aclkinc2_reg);
        dtza_reg     <= (a_hz100g && ckaen_reg && aclkdadj_cry) || (!dtza_res && dtza_reg);
        end
      end
    end

  /*****************************************************************************************/
  /* alarm/clock b                                                                         */
  /*****************************************************************************************/
  assign b_hz100  = ckben_reg && hz100_reg;
  assign b_hz100g = ckben_reg && hz100_reg && !bclkhld_reg;

  assign rd_balrm = chb_sel &&  rdalrm_inst;
  assign rd_bclk  = chb_sel && (rdhclk_inst || rdclk_inst);
  assign rd_bhclk = chb_sel &&  rdhclk_inst;
  assign rd_bonly = rd_bhclk || bclkhld_reg || (rd_bclk && !b_hz100);

  assign wr_balrm = chb_sel &&  wralrm_inst;
  assign wr_bclk  = chb_sel && (wrcclk_inst || wrclk_inst);
  assign wr_bcclk = chb_sel &&  wrcclk_inst;

  assign en_balrm  = b_hz100g || rd_balrm || wr_balrm;
  assign balrm_nxt = (wr_balrm) ? data_reg : balrm_reg[0];
  assign bclk_mat  = b_hz100g &&
                     ~|({bclkdadj_out, bclk_reg[53]} ^ {balrm_reg[0], balrm_reg[55:53]});

  always @ (posedge ph2m or posedge pwr_upb) begin
    if (pwr_upb) begin
      balrm_reg  <= 56'h00000000000000;
      bmatch_reg <= 1'b0;
      almb_reg   <= 1'b0;
      end
    else begin
      if (en_balrm) balrm_reg  <= {balrm_nxt, balrm_reg[55:1]};
      if (ld_cycle) bmatch_reg <= alben_reg && ckben_reg &&
                                  (ld_inst || (bclk_mat && bmatch_reg));
      if (ld_inst)  almb_reg   <= (bclk_mat && bmatch_reg) || (!almb_res && almb_reg);
      end
    end

  assign en_bclk  = b_hz100g || rd_bclk || wr_bclk;
  assign bclk_inc = bclk_reg[0] ^  bclk_cry_in;
  assign bclk_cry = bclk_reg[0] && bclk_cry_in;

  assign bclk_nxt = (wr_bclk)  ? {data_reg,    bclk_reg[55:54]} :
                    (rd_bonly) ? {bclk_reg[0], bclk_reg[55:54]} :
                    (ld_cycle) ?  bclkdadj_out :
                                 {bclk_inc,    bclk_reg[55:54]};

  assign bclk_0cry   = wr_bclk || rd_bonly || !b_hz100g ||
                       ( bclkinc2_reg && b_hz100g && t0_reg); 
  assign bclk_1cry   = (!bclkinc2_reg && b_hz100g && t0_reg) ||
                       ( bclkinc2_reg && b_hz100g && t1_reg);
  assign bclk_cry_in = !bclk_0cry && (bclk_1cry || bclk_cry_reg);

  always @ (bclk_cry or bclk_inc or bclk_reg) begin
    casex ({bclk_cry, bclk_inc, bclk_reg[55:54]})
      4'bx101: bclkdadj_out = 3'b000;                      /* for inc operation            */
      4'bx110: bclkdadj_out = 3'b001;                      /* if result >= 1010            */ 
      4'bx111: bclkdadj_out = 3'b010;                      /* then sub 1010 from result    */
      4'b1000: bclkdadj_out = 3'b011;
      default: bclkdadj_out = {bclk_inc, bclk_reg[55:54]}; /* else no change               */
      endcase
    end

  assign bclkdadj_cry = bclk_cry || (bclk_inc && |bclk_reg[55:54]);

  always @ (posedge ph2m or posedge pwr_upb) begin
    if (pwr_upb) begin
      bclk_cry_reg <= 1'b0;
      bclk_reg     <= 56'h00000000000000;
      bclkhld_reg  <= 1'b0;
      bclkinc2_reg <= 1'b0;
      dtzb_reg     <= 1'b0;
      end
    else begin
      bclk_cry_reg          <= (ld_cycle) ? bclkdadj_cry : bclk_cry;
      if (en_bclk) bclk_reg <= {bclk_nxt, bclk_reg[53:1]};
      if (ld_inst) begin
        bclkhld_reg  <= !wr_bcclk && (rd_bhclk || bclkhld_reg);
        bclkinc2_reg <= (b_hz100 && (rd_bhclk || bclkhld_reg)) ||
                        (!b_hz100g && bclkinc2_reg);
        dtzb_reg     <= (b_hz100g && ckben_reg && bclkdadj_cry) || (!dtzb_res && dtzb_reg);
        end
      end
    end

  /*****************************************************************************************/
  /* interval timer                                                                        */
  /*****************************************************************************************/
  assign i_hz100   = iten_reg && hz100_reg;

  assign en_tmr    = (rdtmr_inst || wrtmr_inst || i_hz100) && state_ec801;
  assign en_tmatch = ld_inst || (ld_cycle && iten_reg && state_ec801);
  assign tmr_nxt   = (wrtmr_inst) ? data_reg : tmr_reg[0];
  assign itmr_tc   = !wrtmr_inst && !itmrtc_reg &&
                     ~|(itmr_nxt ^ {tmr_reg[0], tmr_reg[19:17]});

  always @ (posedge ph2m or posedge pwr_upb) begin
    if (pwr_upb) begin
      tmr_reg    <= 20'h00000;
      tmatch_reg <= 1'b0;
      itmrtc_reg <= 1'b0;
      dtzit_reg  <= 1'b0;
      end
    else begin
      if (en_tmr)    tmr_reg    <= {tmr_nxt, tmr_reg[19:1]};
      if (en_tmatch) tmatch_reg <= iten_reg && (ld_inst || (itmr_tc && tmatch_reg));
      if (ld_inst)   itmrtc_reg <=  i_hz100 && tmatch_reg;
      if (ld_inst)   dtzit_reg  <= (i_hz100 && tmatch_reg) || (!dtzit_res && dtzit_reg);
      end
    end

  assign en_itmr  = (wrtmr_inst || itmrtc_reg || i_hz100) && state_ec801;
  assign itmr_inc = itmr_reg[0] ^  itmr_cry_reg;
  assign itmr_cry = itmr_reg[0] && itmr_cry_reg;
  assign itmr_nxt = (wrtmr_inst) ? 4'b0000 :
                    (itmrtc_reg) ? 4'b0000 :
                    (ld_cycle)   ? {itmrdadj_out, itmr_reg[17]} :
                                   {itmr_inc,     itmr_reg[19:17]};

  always @ (itmr_cry or itmr_inc or itmr_reg) begin
    casex ({itmr_cry, itmr_inc, itmr_reg[19:18]})
      4'bx101: itmrdadj_out = 3'b000;                      /* for inc operation            */
      4'bx110: itmrdadj_out = 3'b001;                      /* if result >= 1010            */ 
      4'bx111: itmrdadj_out = 3'b010;                      /* then sub 1010 from result    */
      4'b1000: itmrdadj_out = 3'b011;
      default: itmrdadj_out = {itmr_inc, itmr_reg[19:18]}; /* else no change               */
      endcase
    end

  assign itmrdadj_cry = itmr_cry || (itmr_inc && |itmr_reg[19:18]);

  always @ (posedge ph2m or posedge pwr_upb) begin
    if (pwr_upb) begin
      itmr_cry_reg <= 1'b0;
      itmr_reg     <= 20'h00000;
      end
    else begin
      itmr_cry_reg <= (ld_cycle) ? ((iten_reg && ld_inst) || itmrdadj_cry) : itmr_cry;
      if (en_itmr) itmr_reg <= {itmr_nxt, itmr_reg[16:1]};
      end
    end

  /*****************************************************************************************/
  /* accuracy factor                                                                       */
  /*****************************************************************************************/
  assign x_hz100     = accen_reg && hz100_reg;

  assign en_acc      = (rdacc_inst || wracc_inst || x_hz100) && state_c80p;
  assign en_accmatch = ld_inst || (ld_cycle && accen_reg && state_c80);
  assign acc_nxt     = (wracc_inst) ? data_reg : acc_reg[0];
  assign acctmr_tc   = !wracc_inst && !acctmrtc_reg &&
                       ~|(acctmr_nxt ^ {acc_reg[0], acc_reg[12:10]});

  always @ (posedge ph2m or posedge pwr_upb) begin
    if (pwr_upb) begin
      acc_reg      <= 13'h0000;
      accen_reg    <= 1'b0;
      accmatch_reg <= 1'b0;
      acctmrtc_reg <= 1'b0;
      updown_reg   <= 1'b0;
      end
    else begin
      if (en_acc)      acc_reg      <= {acc_nxt, acc_reg[12:1]};
      if (en_accmatch) accmatch_reg <= accen_reg && (ld_inst || (acctmr_tc && accmatch_reg));
      if (ld_inst) begin
        accen_reg    <= |acc_reg[11:0];
        acctmrtc_reg <= x_hz100 && accmatch_reg;
        updown_reg   <= acc_reg[12];
        end
      end
    end

  assign en_acctmr  = (wracc_inst || acctmrtc_reg || x_hz100) && state_ec80;
  assign acctmr_inc = acctmr_reg[0] ^  acctmr_cry_reg;
  assign acctmr_cry = acctmr_reg[0] && acctmr_cry_reg;
  assign acctmr_nxt = (wracc_inst)   ? 4'b0000 :
                      (acctmrtc_reg) ? 4'b0000 :
                      (ld_cycle)     ? {acctmrdadj_out, acctmr_reg[13]} :
                                       {acctmr_inc,     acctmr_reg[15:13]};

  always @ (acctmr_cry or acctmr_inc or acctmr_reg) begin
    casex ({acctmr_cry, acctmr_inc, acctmr_reg[15:14]})
      4'bx101: acctmrdadj_out = 3'b000;                    /* for inc operation            */
      4'bx110: acctmrdadj_out = 3'b001;                    /* if result >= 1010            */ 
      4'bx111: acctmrdadj_out = 3'b010;                    /* then sub 1010 from result    */
      4'b1000: acctmrdadj_out = 3'b011;
      default: acctmrdadj_out = {acctmr_inc, acctmr_reg[15:14]};
      endcase
    end

  assign acctmrdadj_cry = acctmr_cry || (acctmr_inc && |acctmr_reg[15:14]);

  always @ (posedge ph2m or posedge pwr_upb) begin
    if (pwr_upb) begin
      acctmr_cry_reg <= 1'b0;
      acctmr_reg     <= 16'h0000;
      end
    else begin
      acctmr_cry_reg <= (ld_cycle) ? ((accen_reg && ld_inst) || acctmrdadj_cry) : acctmr_cry;
      if (en_acctmr) acctmr_reg <= {acctmr_nxt, acctmr_reg[12:1]};
      end
    end

  endmodule





