/*******************************************************************************************/
/**                                                                                       **/
/** COPYRIGHT (C) 2018, Systemyde International Corporation, ALL RIGHTS RESERVED          **/
/**                                                                                       **/
/** timer board                                                       Rev 0.0  08/04/2018 **/
/**                                                                                       **/
/*******************************************************************************************/
module time_brd (data_bus, isa_bus, fi_bus, osc_in, ph1, ph2, pwr_up, pwo, sync);

  inout         data_bus;      /* memory/peripheral data bus                          bus  */
  inout         isa_bus;       /* instruction address/data bus                        bus  */
  input         osc_in;        /* 32768Hz clock input                               32khz  */
  input         ph1;           /* clock phase 1                                       bus  */
  input         ph2;           /* clock phase 2                                       bus  */
  input         pwr_up;        /* power on reset                                      por  */
  input         pwo;           /* power off                                           bus  */
  input         sync;          /* synchronization                                     bus  */
  output        fi_bus;        /* flag bus                                            bus  */

  /*****************************************************************************************/
  /*                                                                                       */
  /* signal declarations                                                                   */
  /*                                                                                       */
  /*****************************************************************************************/
  wire        data_drv;                                    /* memory/peripheral data bus   */
  wire        data_out;                                    /* memory/peripheral data bus   */
  wire        fi_bus;                                      /* flag bus                     */
  wire        fi_drv;                                      /* flag bus                     */
  wire        fi_out;                                      /* flag bus                     */
  wire        isa_drv;                                     /* drive isa_bus                */
  wire        isa_out;                                     /* data: isa_bus                */

  /*****************************************************************************************/
  /*                                                                                       */
  /* bus drivers                                                                           */
  /*                                                                                       */
  /*****************************************************************************************/
  assign data_bus = (data_drv) ? data_out : 1'bz;
  assign isa_bus  = (isa_drv)  ? isa_out  : 1'bz;
  assign fi_bus   = (fi_drv)   ? fi_out   : 1'bz;

  /*****************************************************************************************/
  /*                                                                                       */
  /* fpga                                                                                  */
  /*                                                                                       */
  /*****************************************************************************************/
  time_top FPGA ( .data_drv(data_drv), .data_out(data_out), .fi_drv(fi_drv), .fi_out(fi_out),
                  .isa_drv(isa_drv), .isa_out(isa_out), .spi_csn(), .spi_sck(),
                  .spi_so(), .data_bus(data_bus), .isa_bus(isa_bus), .osc_in(osc_in),
                  .ph1(ph1), .ph2(ph2), .pwr_up(pwr_up), .pwo(pwo), .spi_si(1'b0),
                  .sync(sync) );

  endmodule





